%Function:  Import the raw data and put them into a big array
%Param:     dataFileName_       The text file containing the raw data
%           timeGap_            The time gap used to filter the data
%           flagDrawFigure_     0, don't draw figure in the function
%                               not 0, draw it
%Return:    indoor_temp_array       The indoor temperature data array
%           outdoor_temp_array      The outdoor temperature data array
%           energy_array            The time array
%           
%Array format: time, node1~node10, node15
function [indoor_temp_array, outdoor_temp_array, energy_array]=Fn_ImportRawData(dataFileName_, timeGap_, flagDrawFigure_)
TIME_ALIGN_SIZE = 1000*60;   %Align at 1 min

%Import real data
fid = fopen(dataFileName_);
if(fid == -1)
    Aligned_array_size = -1;
    fprintf('Cannot open file: %s\n',dataFileName_);
    return;
end
C = textscan(fid, '%d %f %s %f');
fclose(fid);

%Calculate the array size
array_size = size(C{1},1);
R_TimeArr = C{4}';
Aligned_array_size = 1+fix((R_TimeArr(array_size) - R_TimeArr(1))/TIME_ALIGN_SIZE);
R_NodeArr = C{1}';
R_ValueArr = C{2}';
R_TimeStrArr = C{3}';

Aligned_data_array = zeros(Aligned_array_size, 11);     %Used to store data of nodes
Aligned_time_array = cell(1, Aligned_array_size);      %Used to store the time stamps
R_array_index = 1;

Aligned_array_index = 1;
while Aligned_array_index < Aligned_array_size
    haveDataArr = zeros(1,11);
    
    if(R_array_index > array_size)
        Aligned_array_size = Aligned_array_index-1; %Adjust the array size;
        break;
    end
    Aligned_array_index_increase = 0;
    Time_Align = fix(R_TimeArr(R_array_index)/TIME_ALIGN_SIZE);
    
    %If still in the time range
    while ((Time_Align+timeGap_) > fix(R_TimeArr(R_array_index)/TIME_ALIGN_SIZE))
        
        node_num = R_NodeArr(R_array_index);
        if(node_num == 15)
            node_num = 11;
        end
        if(node_num == 0)
            R_array_index = R_array_index+1;
            break;
        end
        Aligned_array_index_increase = 1;
        if(haveDataArr(node_num) == 0)
            %If the node data is not filled, fill it.

            haveDataArr(node_num) = 1;
            Aligned_data_array(Aligned_array_index, node_num) = R_ValueArr(R_array_index);
            Aligned_time_array(Aligned_array_index) = R_TimeStrArr(R_array_index);
            if node_num ~=11
                %Sometimes the data is wrong, the temperature should not be
                %less than 10
               if R_ValueArr(R_array_index)<10
                   haveDataArr(node_num) = 0;
               end
            end
        end
        R_array_index = R_array_index+1;
        if(R_array_index > array_size)
            break;
        end
    end
    
    %if there is some zero data in this column, drop it
    skip_this_column = 0;
    for index = 1:11
        if haveDataArr(index)==0
            if(index ~= 11)
                skip_this_column = 1;
                break;    
            end
            
        end
    end
    if skip_this_column == 1
        continue;
    end
    
    Aligned_array_index = Aligned_array_index+Aligned_array_index_increase;
end
Aligned_data_array = Aligned_data_array(1:(Aligned_array_index-1),:);
Aligned_data_array = Aligned_data_array';


indoor_temp_array = Aligned_data_array(1:9, :);
%indoor_temp_array = Aligned_data_array(9, :);
outdoor_temp_array = Aligned_data_array(10,:);
energy_array = Aligned_data_array(11, :);

if flagDrawFigure_ == 0
    return;
end

Style_arr = {'-k','--k',':k','-b','--b',':b','-m','--m',':m','-r' };
t = 1:timeGap_:Aligned_array_size*timeGap_;
hold on
for index = 1:10
    plot(t,Aligned_data_array(index,1:Aligned_array_size),char(Style_arr{index}));
end
hold off

end
