%Function:  Import the raw data and put them into a big array
%Param:     dataFileName_       The text file containing the raw data
%           timeGap_            The time gap used to filter the data
%           flagDrawFigure_     0, don't draw figure in the function
%                               not 0, draw it
%Return:    indoor_temp_array       The indoor temperature data array
%           outdoor_temp_array      The outdoor temperature data array
%           energy_array            The time array
%           
%Array format: time, node1~node10, node15
function [indoor_temp_array, outdoor_temp_array, energy_array]=Fn_ImportSemiProcessedData(dataFileName_, timeGap_, flagDrawFigure_)


%Import real data
fid = fopen(dataFileName_);
if(fid == -1)
    Aligned_array_size = -1;
    fprintf('Cannot open file: %s\n',dataFileName_);
    return;
end
C = textscan(fid, '%d %d %f %f %f %f %f %f %f %f %f %f');
fclose(fid);

%Calculate the array size
array_size = size(C{1},1);
R_Hour = C{1}';
R_Minute = C{2}';
R_Energy = C{3}';
for i = 1:9
    R_Temp(i,:) = C{3+i}';
end

%Reverse the time sequence
for i = 1: array_size/2
    % Exchange hour, minute and energy
    temp_hour = R_Hour(i);
    R_Hour(i) = R_Hour(array_size+1-i);
    R_Hour(array_size+1-i) = temp_hour;
    temp_minute = R_Minute(i);
    R_Minute(i) = R_Minute(array_size+1-i);
    R_Minute(array_size+1-i) = temp_minute;
    temp_energy = R_Energy(i);
    R_Energy(i) = R_Energy(array_size+1-i);
    R_Energy(array_size+1-i) = temp_energy;
   
    % Exchange temp
    temp_temp = R_Temp(1:9, i);
    R_Temp(1:9, i) = R_Temp(1:9, array_size+1-i);
    R_Temp(1:9, array_size+1-i) = temp_temp;
end

for i = 1:9
    for j = 1:array_size
        if R_Temp(i, j)<10 || R_Temp(i,j) >40 
            R_Temp(i,j) = (R_Temp(i,j-1)+R_Temp(i, j+1))/2;
        end
    end
end

indoor_temp_array = R_Temp(1:8,:);
outdoor_temp_array = R_Temp(9,:);
energy_array = R_Energy;

if flagDrawFigure_ == 0
    return;
end

Style_arr = {'-k','--k',':k','-b','--b',':b','-m','--m',':m','-r' };
t = 1:timeGap_:Aligned_array_size*timeGap_;
hold on
for index = 1:10
    plot(t,Aligned_data_array(index,1:Aligned_array_size),char(Style_arr{index}));
end
hold off

end
