#include "stdio.h"
#include "DPScheduleAlg.h"

DPNode::~DPNode()
{
	assert(m_inRefNum == 0);
	// delete the edges, the edges will delete the end node automatically
	for(int i=0; i< m_edgeNum; i++)
	{
		delete m_outEdges[i];
	}

	delete[] m_outEdges;
};


int DPScheduleAlg::initAlg(MeetingInfo_ptr* meetingEndTimeArr, int meetingNum, RoomInfo* roomInfoArr, int roomNum)
{
	m_endTimeArr = meetingEndTimeArr;
	m_meetingNum = meetingNum;
	m_roomInfoArr = roomInfoArr;
	m_roomNum = roomNum;

	m_skylineHeaderArr = new SkyLineNode_ptr[m_meetingNum];
	m_skylineNumArr = new int[m_meetingNum];
	for(int i = 0; i<m_meetingNum; i++)
	{
		m_skylineNumArr[i] = 0;
		m_skylineHeaderArr[i] = NULL;
	}
	
	int* skyline_time = new int[m_roomNum];
	for(int i = 0; i<m_roomNum; i++)
	{
		skyline_time[i] = 0;
	}

	// Init the first stage
	m_graphSource = new SkyLineNode(skyline_time, m_roomNum, -1);
	m_graphDestination = new SkyLineNode(skyline_time, m_roomNum, -1);

	
	extendSkyline(m_graphSource,0);
	printf("meeting index: %d, \tskyline number:%d\n", 0,  m_skylineNumArr[0]);
	// Construct the tree
	SkyLineNode_ptr chain_iterator = NULL;
	for(int meeting_index = 0; meeting_index<(m_meetingNum-1); meeting_index++)
	{
		
		chain_iterator = m_skylineHeaderArr[meeting_index];
		while(chain_iterator != NULL)
		{
			extendSkyline(chain_iterator, meeting_index+1);
			chain_iterator = chain_iterator->nextSkyLine();
		}
		printf("meeting index: %d, \tskyline number:%d\n", meeting_index+1,  m_skylineNumArr[meeting_index+1]);
	}

	//Init the final stage
	chain_iterator = m_skylineHeaderArr[m_meetingNum-1];

	//connect to destination
	while(chain_iterator != NULL)
	{
		ScheduleEdge_ptr p_edge = new ScheduleEdge(chain_iterator, m_graphDestination, 0, NULL, NULL);
		chain_iterator = chain_iterator->nextSkyLine();
	}
	
	
	delete[] skyline_time;
	return 0;
}


int DPScheduleAlg::destroy()
{
	if(m_isDestroyed != 0)
		return 0;
	m_isDestroyed = 1;
	
	delete m_graphSource;
	SkyLineNode_ptr chain_iterator = NULL;

	for(int i=0; i< m_meetingNum; i++)
	{
		chain_iterator = m_skylineHeaderArr[i];
		while(chain_iterator != NULL)
		{
			SkyLineNode_ptr temp_iterator = chain_iterator->nextSkyLine();	
			delete chain_iterator;
			chain_iterator = temp_iterator;
		}
	}
	//m_graphDestination is deleted in the loop;
	
	if(m_skylineHeaderArr != NULL)
		delete[] m_skylineHeaderArr;
	if(m_skylineNumArr != NULL)
		delete[] m_skylineNumArr;

	
	return 0;
}

int DPScheduleAlg::schedule()
{
	DPNode_ptr chain_header = NULL;
	DPNode_ptr chain_tail = NULL;
	m_graphSource->dpValue(0);
	chain_header = m_graphSource;
	chain_header->isInStackChain(1);
	chain_tail = chain_header;
	while(chain_header != NULL)
	{
		int edge_num = chain_header->outEdgeNum();
		Edge_ptr* out_edge_arr = chain_header->outEdges();
		
		float current_dp_value = chain_header->dpValue();

		for(int i=0; i< edge_num; i++)
		{
			int f_add_to_stack = 0;
			Edge_ptr current_edge = out_edge_arr[i];
			float edge_value = current_edge->edgeValue();
			float tail_node_value;
			DPNode_ptr edge_tail_node = current_edge->tailNode();
			tail_node_value = edge_tail_node->dpValue();

			
			if(tail_node_value == INIT_DP_VALUE)
			{
				edge_tail_node->dpValue(current_dp_value+edge_value);
				edge_tail_node->dpEdge(current_edge);
				f_add_to_stack = 1;
			}
			else
			{
				if((current_dp_value+edge_value)<tail_node_value)
				{
					edge_tail_node->dpValue(current_dp_value+edge_value);
					edge_tail_node->dpEdge(current_edge);
					f_add_to_stack = 1;
				}
			}

			if((f_add_to_stack == 1)&&!(edge_tail_node->isInStackChain()))
			{
				edge_tail_node->isInStackChain(1);
				edge_tail_node->nextStackNode(NULL);
				chain_tail->nextStackNode(edge_tail_node);
				chain_tail = edge_tail_node;
			}
		}

		//Pop a node out
		chain_header->isInStackChain(0);
		chain_header=chain_header->nextStackNode();
		
	}

	//Print the schedule
	ScheduleEdge_ptr schedule_edge = static_cast<ScheduleEdge_ptr>(m_graphDestination->dpEdge());
	SkyLineNode_ptr schedule_skyline = static_cast<SkyLineNode_ptr>(schedule_edge->headNode());
	schedule_edge = static_cast<ScheduleEdge_ptr>(schedule_skyline->dpEdge());
	while(schedule_edge!= NULL)
	{
		MeetingInfo_ptr meeting = schedule_edge->meeting();
		RoomInfo_ptr room = schedule_edge->room();
		printf("Meeting %s %d-%d in room %s\n",meeting->meetingName(), meeting->startTime(), meeting->endTime(), room->roomName());

		schedule_skyline = static_cast<SkyLineNode_ptr>(schedule_edge->headNode());
		schedule_edge = static_cast<ScheduleEdge_ptr>(schedule_skyline->dpEdge());
	};
	return 0;
}

float DPScheduleAlg::calculateEnergy()
{
	
	return m_graphDestination->dpValue();
}


int DPScheduleAlg::extendSkyline(SkyLineNode_ptr headSkyline,int nextSkylineIndex)
{
	MeetingInfo_ptr current_meeting = m_endTimeArr[nextSkylineIndex];
	
	int* skyline_time = headSkyline->endTimeArr(); //NOTICE: This end time array should be recovered at the end of this function
	for(int room_index=0; room_index<m_roomNum; room_index++)
	{
		int old_time = skyline_time[room_index];

		//Make sure this is a suitable room
		if(old_time > current_meeting->startTime())
		{
			continue;
		}
		if(m_roomInfoArr[room_index].capacity() < current_meeting->capacityRequirement())
		{
			continue;
		}

		//Begin to add the node and edge
		skyline_time[room_index] = current_meeting->endTime();
		SkyLineNode_ptr p_skyline = NULL;

		//Search the skyline chain to avoid adding same skyline.
		SkyLineNode_ptr p_next_chain_skyline = m_skylineHeaderArr[nextSkylineIndex];
		SkyLineNode_ptr p_chain_tail = NULL;
		int f_add_chain_skyline_node= 1;
		while( p_next_chain_skyline != NULL )
		{
			if(p_next_chain_skyline->isSameSkyLine(skyline_time))
			{
				p_skyline = p_next_chain_skyline;
				f_add_chain_skyline_node = 0;
				break;
			}
			p_chain_tail = p_next_chain_skyline;
			p_next_chain_skyline = p_chain_tail->nextSkyLine();
		}

		if(p_skyline == NULL)
			p_skyline = new SkyLineNode(skyline_time, m_roomNum, nextSkylineIndex);

		
		float energy_value = m_roomInfoArr[room_index].calculateMeetingEnergy(old_time, current_meeting);
		ScheduleEdge_ptr p_edge = new ScheduleEdge(headSkyline, p_skyline, energy_value, &m_roomInfoArr[room_index], current_meeting);

		//Put them into the skyline chain
		if(f_add_chain_skyline_node != 0)
		{
			if(p_chain_tail == NULL)
			{
				m_skylineHeaderArr[nextSkylineIndex] = p_skyline;
				p_chain_tail = p_skyline;
			}
			else
			{
				p_chain_tail->nextSkyLine(p_skyline);
			}
			m_skylineNumArr[nextSkylineIndex]++;
		}
		
		//Recover the skyline time.
		skyline_time[room_index] = old_time;
	}

	return 0;
}
