
#ifndef __DP_SCHEDULE_ALG_H__
#define __DP_SCHEDULE_ALG_H__

#include <assert.h>
#include "MeetingInfo.h"
#include "GroupInfoManager.h"

#define INIT_DP_VALUE -1
class Edge;
typedef Edge* Edge_ptr;

class DPNode;
typedef DPNode* DPNode_ptr;

class DPNode{
	//Common dynamice programming members
	float m_dpValue;
	Edge_ptr m_dpEdge;

	int m_edgeNum;
	int m_maxEdgeNum;
	Edge_ptr* m_outEdges;

	int m_inRefNum;

	DPNode_ptr m_nextStackNode;
	int m_isInStackChain;
	
public:
	DPNode(int maxEdgeNum, float dpValue):
		m_edgeNum(0),m_maxEdgeNum(maxEdgeNum), m_dpValue(dpValue), m_dpEdge(NULL), m_inRefNum(0), m_nextStackNode(NULL), m_isInStackChain(0){
		m_outEdges = new Edge_ptr[m_maxEdgeNum];
	};
	~DPNode();

	int addOutEdge(Edge_ptr outEdge)
	{
		if(m_edgeNum >= m_maxEdgeNum)
		{
			// TODO: Error
			return 0;
		}

		m_outEdges[m_edgeNum] = outEdge;
		m_edgeNum++;
		return m_edgeNum;
	}

	int addInRefNum(){ return m_inRefNum++;}
	int delInRefNum(){ m_inRefNum--; return m_inRefNum;}

	void dpValue(float value){m_dpValue = value;}
	float dpValue(){return m_dpValue;};
	
	void dpEdge(Edge_ptr edge){ m_dpEdge = edge;}
	Edge_ptr dpEdge(){return m_dpEdge;}

	void nextStackNode(DPNode_ptr nextNode){m_nextStackNode = nextNode;}
	DPNode_ptr nextStackNode(){return m_nextStackNode;}

	void isInStackChain(int isIn){m_isInStackChain = isIn;}
	int isInStackChain(){return m_isInStackChain;}

	Edge_ptr* outEdges(){return m_outEdges;}
	int outEdgeNum(){return m_edgeNum;}
	
};

class Edge
{
	float m_edgeValue;
	DPNode_ptr m_headNode;
	DPNode_ptr m_tailNode;
	
public:
	Edge(DPNode_ptr headNode, DPNode_ptr tailNode, float edgeValue):m_edgeValue(edgeValue),m_headNode(headNode),m_tailNode(tailNode){
		assert(headNode!=NULL);
		assert(tailNode != NULL);

		//Make changes to the header and tail
		m_tailNode->addInRefNum();
		m_headNode->addOutEdge(this);
	};
	~Edge()
	{
		//Try to delete the tail node
		m_tailNode->delInRefNum();		
	}

	DPNode_ptr headNode(){return m_headNode;};
	DPNode_ptr tailNode(){return m_tailNode;};

	float edgeValue(){return m_edgeValue;};
};

class SkyLineNode;
typedef SkyLineNode* SkyLineNode_ptr;


class SkyLineNode:public DPNode
{
	int* m_endTimeArr;
	int m_roomNum;
	int m_index;

	SkyLineNode_ptr m_nextSkyLine;	//Used for connecting the skylines in the same i;

	
public:
	SkyLineNode(int * endTimeArr, int roomNum, int index):m_roomNum(roomNum),m_index(index),m_nextSkyLine(NULL), DPNode(roomNum, INIT_DP_VALUE){
		//Create and copy
		m_endTimeArr = new int[m_roomNum];
		for(int i=0; i< m_roomNum; i++)
		{
			m_endTimeArr[i] = endTimeArr[i];
		}
	};
	
	~SkyLineNode()
	{
		delete m_endTimeArr;
	};

	void nextSkyLine(SkyLineNode_ptr skyLine){m_nextSkyLine = skyLine;}
	SkyLineNode_ptr nextSkyLine(){return m_nextSkyLine;}

	int roomNum(){return m_roomNum;}
	int* endTimeArr(){return m_endTimeArr;};
#if 0
	int isSameSkyLine(SkyLineNode_ptr skyLine)
	{
		if(m_roomNum != skyLine->roomNum())
			return 0;
		
		const int* end_time_arr = skyLine->endTimeArr();
		for(int i=0; i< m_roomNum; i++)
		{
			if(m_endTimeArr[i]!=end_time_arr[i])
				return 0;
		}

		return 1;
		
	}
#endif
	int isSameSkyLine(int* skelineTime)
	{
		for(int i=0; i< m_roomNum; i++)
		{
			if(m_endTimeArr[i]!=skelineTime[i])
				return 0;
		}
		return 1;
	}
};

class ScheduleEdge;
typedef ScheduleEdge* ScheduleEdge_ptr;

class ScheduleEdge:public Edge
{
private:
	RoomInfo_ptr m_room;
	MeetingInfo_ptr m_meeting;
public:
	ScheduleEdge(DPNode_ptr headNode, DPNode_ptr tailNode, float edgeValue, RoomInfo_ptr room, MeetingInfo_ptr meeting):	
		Edge(headNode, tailNode, edgeValue), m_room(room),m_meeting(meeting){
	};
	~ScheduleEdge()
	{
	}

	RoomInfo_ptr room(){return m_room;}
	MeetingInfo_ptr meeting(){return m_meeting;}
};

class DPScheduleAlg{

	MeetingInfo_ptr* m_endTimeArr;
	int m_meetingNum;

	RoomInfo* m_roomInfoArr;
	int m_roomNum;

	SkyLineNode_ptr* m_skylineHeaderArr;
	int* m_skylineNumArr;

	SkyLineNode_ptr m_graphSource, m_graphDestination;
	int m_isDestroyed;
public:
	DPScheduleAlg():
		m_endTimeArr(NULL), m_meetingNum(0), m_roomInfoArr(NULL), m_roomNum(0),
		m_skylineHeaderArr(NULL), m_skylineNumArr(0),m_graphSource(NULL),m_graphDestination(NULL),m_isDestroyed(0){};
		
	~DPScheduleAlg()
	{
		// TODO: Delete the tree
		destroy();
	};

	int initAlg(MeetingInfo_ptr* meetingEndTimeArr, int meetingNum, RoomInfo* roomInfoArr, int roomNum);

	int destroy();


	int schedule();

	float calculateEnergy();

	
private:

	int extendSkyline(SkyLineNode_ptr headSkyline, int nextSkylineIndex);
};

#endif

