

#ifndef __MEETING_INFO_H__
#define __MEETING_INFO_H__

#include "string.h"
#include "math.h"
#include "stdio.h"


#define MEETING_START_TIME 8*60
#define MEETING_END_TIME 23*60

class OutdoorTemperature{
	float m_outdoorTemperatureArr[MEETING_END_TIME-MEETING_START_TIME];
	int m_numOfData;
	int m_dataStep;
public:
	OutdoorTemperature(){}

	//filename	The name of outdoor temperature file
	//step		Time interval between data in the file, (minute)
	int init(const char* filename, int step)
	{
		m_dataStep = step;

		FILE* fp;
		fp = fopen(filename, "r");
		if(fp == NULL)
		{
			printf("Can not open file %s\n", filename);
			return 0;
		}	

		int ret;
		float outdoor_temperature;

		//Default step
		m_numOfData = 0;
		for(int i = 0; i<MEETING_END_TIME-MEETING_START_TIME; i++)
		{
			ret = fscanf(fp, "%f\n", &outdoor_temperature);
			if(ret != 1)
				break;

			m_outdoorTemperatureArr[i] = outdoor_temperature;
			m_numOfData++;
		}
		
	}
	float temperatureAt(int time)
	{
		int index = (time-MEETING_START_TIME)/m_dataStep;
		return m_outdoorTemperatureArr[time-MEETING_START_TIME];
	}
};

static OutdoorTemperature* g_outdoorTemp;

class RoomInfo;
typedef RoomInfo* RoomInfo_ptr;

class MeetingInfo;
typedef MeetingInfo* MeetingInfo_ptr;

class MeetingInfo{
	char m_meetingName[20];
	int m_startTime;
	int m_endTime;
	int m_capacityReq;
	int m_facilityReq;

	char m_roomName[20];

	RoomInfo_ptr m_room;
	int m_scheduled;
	
public:
	MeetingInfo():m_scheduled(0){};
	~MeetingInfo(){};
	const char* meetingName(){return m_meetingName;}
	int meetingName(const char* meeting_name){ 
		strcpy(m_meetingName, meeting_name);
		return 0;
	}

	int startTime(){return m_startTime;};
	void startTime(int startTime){m_startTime = startTime;}


	int endTime(){return m_endTime;};
	void endTime(int endTime){m_endTime = endTime;}

	int capacityRequirement(){return m_capacityReq;}
	void capacityRequirement(int capacityReq){ m_capacityReq = capacityReq;}

	int facilityRequirement(){return m_facilityReq;}
	void facilityRequirement(int facilityReq){m_facilityReq = facilityReq;}
	
	const char* roomName(){return m_roomName;}
	int roomName(const char* roomName)
	{
		strcpy(m_roomName, roomName);
		return 0;
	}

	RoomInfo_ptr room(){return m_room;}
	int room(RoomInfo_ptr room)
	{
		m_room = room;
		m_scheduled = 1;
		return 0;
	}
	
	void resetSchedule()	{m_scheduled = 0;}
	int isScheduled(){return m_scheduled;}
	
	int isSameMeeting(MeetingInfo_ptr meetingInfo)
	{
		const char* room_name = meetingInfo->roomName();
		int start_time = meetingInfo->startTime();
		int end_time = meetingInfo->endTime();
		
		if((strcmp(m_roomName, room_name)==0)&&(m_startTime==start_time)&&(m_endTime == end_time))
			return 1;
		return 0;
	}
	
	MeetingInfo& operator=(MeetingInfo_ptr meetingInfo)
	{
		this->meetingName(meetingInfo->meetingName());
		this->startTime(meetingInfo->startTime());
		this->endTime(meetingInfo->endTime());
		this->capacityRequirement(meetingInfo->capacityRequirement());
		this->roomName(meetingInfo->roomName());
		return *this;
	}
};

#define SCHEDULE_PURE 0
#define SCHEDULE_DUMMY 1
#define SCHEDULE_WITHOUT_RECOOL 2


class RoomInfo
{
private:
	char m_roomName[20];
	unsigned short m_capacity;
	unsigned short m_facility;	//We consider one facility

	MeetingInfo_ptr m_meetingSchedule[20];
	int m_scheduledMeetingNum;

	float m_height;
	float m_width;
	float m_length;

	float m_x;
	float m_y;
	float m_z;

#define AC_TRANS_RATE -0.14
#define OUTSIDE_TEMP 25
#define TARGET_TEMP 20
#define kL  0.53
	float m_ACPower;
	float m_effectivePower;
	float m_keepPower;
	float m_lambdaA;
	float m_mC;

public:
	RoomInfo():m_capacity(0), m_scheduledMeetingNum(0), m_height(0), m_width(0), m_length(0){};
	~RoomInfo()
	{ 
	};

	const char* roomName(){return m_roomName;}
	const char* roomName(const char* roomName){
		strcpy(m_roomName, roomName);
		return m_roomName;
	}

	unsigned short capacity(){return m_capacity;}
	void capacity(unsigned short capacity)
	{
		m_capacity = capacity;
	}

	void facility(unsigned short facility){m_facility = facility;}
	unsigned short facility(){return m_facility;}
	
	bool isFacilitySatisfied(unsigned short facilityReq){return facilityReq < m_facility;}
	
	void roomSizeAndACPower(float length, float width, float height, float ACPower)
	{
		m_length = length;
		m_width = width;
		m_height = height;
		m_ACPower = ACPower;

		float area = (m_width+m_length)*2*m_height+ m_width*m_length*2;
		m_lambdaA = area*kL;
		m_mC = (m_height*m_length*m_width)*1.2*1005; //60? C=1.005kJ/(kg*K)
		m_keepPower = (TARGET_TEMP-OUTSIDE_TEMP)*m_lambdaA/AC_TRANS_RATE;
		m_effectivePower = m_ACPower*AC_TRANS_RATE;
		
	}

	float length(){return m_length;}
	float width(){return m_width;}
	float height(){return m_height;}
	float ACPower(){return m_ACPower;}

	void resetSchedule()
	{
		for(int i=0; i< m_scheduledMeetingNum; i++)
		{
			m_meetingSchedule[i]->resetSchedule();
			m_meetingSchedule[i]=NULL;
		}
		m_scheduledMeetingNum = 0;
	}

	//If time conflicts, return -1
	int isMeetingConflict(MeetingInfo_ptr meeting)
	{
		int meeting_st = meeting->startTime();
		int meeting_et = meeting->endTime();
		MeetingInfo_ptr current_meeting;
		int cur_st, cur_et;
		for(int i=0; i<m_scheduledMeetingNum; i++)
		{
			current_meeting = m_meetingSchedule[i];
			cur_st = current_meeting->startTime();
			cur_et = current_meeting->endTime();

			//If no conflict, continue;
			if((meeting_et<=cur_st)||(meeting_st>=cur_et))
				continue;
			else
				return -1;
		}
		return 0;
	}
	
	//If have time conflict, return -1;
	int putMeeting(MeetingInfo_ptr meeting)
	{

		if(-1 == isMeetingConflict(meeting))
			return -1;
		
		m_meetingSchedule[m_scheduledMeetingNum] = meeting;
		m_scheduledMeetingNum++;
		meeting->room(this);
		return 0;
	}

	// Try to insert a meeting
	// Return: -1 if failed.
	//		  0	if success.
	int insertMeeting(MeetingInfo_ptr meeting)
	{

		int insert_location = -1;

		int start_time = meeting->startTime();
		int end_time = meeting->endTime();

		//Check whether insertable
		if(start_time >= lastMeetingEndTime())
			return -1;

		if(m_scheduledMeetingNum <=1)
			return -1;
		
		
		for(int i= 0; i < (m_scheduledMeetingNum-1); i++)
		{
			int next_start_time = m_meetingSchedule[i+1]->startTime();
			int current_end_time = m_meetingSchedule[i]->endTime();
			//Check whether current meeting can be inserted.
			if((current_end_time<= start_time)&&(end_time<= next_start_time))
			{
				insert_location = i;
				break;
			}
		}
		//Check search result
		if(insert_location == -1)
			return -1;

		//Insert meeting and add scheduled meeting number
		for(int i = m_scheduledMeetingNum-1; i>insert_location; i--)
		{
			m_meetingSchedule[i+1] = m_meetingSchedule[i];
		}
		meeting->room(this);		//Meeting scheduled
		m_meetingSchedule[insert_location+1] = meeting;
		m_scheduledMeetingNum++;

		return 0;
	}
	
	int lastMeetingEndTime()
	{ 
		if(m_scheduledMeetingNum>0)
			return m_meetingSchedule[m_scheduledMeetingNum-1]->endTime();
		return MEETING_START_TIME;	//Start at 8 clock
	}

	int isSameRoom(RoomInfo& roomInfo)
	{
		if(strcmp(m_roomName, roomInfo.roomName())==0)
			return 1;
		return 0;
	}
	
	RoomInfo& operator=(RoomInfo& roomInfo)
	{
		this->capacity(roomInfo.capacity());
		this->roomName(roomInfo.roomName());
		this->roomSizeAndACPower(roomInfo.length(),roomInfo.width(),roomInfo.height(),roomInfo.ACPower());
		
		return *this;
	}

	float calculateRoomEnergy()
	{
		float energy_consumption= 0;
		MeetingInfo_ptr last_meeting = NULL;
		for(int i =0; i<m_scheduledMeetingNum; i++)
		{
			energy_consumption += calculateMeetingEnergy(last_meeting, m_meetingSchedule[i]);
			last_meeting = m_meetingSchedule[i];
		}
		return energy_consumption;
		
	}

	float calculateRoomMinEnergy()
	{
		float energy_consumption= 0;
		MeetingInfo_ptr last_meeting = NULL;
		for(int i =0; i<m_scheduledMeetingNum; i++)
		{
			energy_consumption += calculateMeetingMinEnergy(last_meeting, m_meetingSchedule[i]);
			last_meeting = m_meetingSchedule[i];
		}
		return energy_consumption;		
	}

	
	float calculateRoomMaxEnergy()
	{
		float energy_consumption= 0;
		MeetingInfo_ptr last_meeting = NULL;
		for(int i =0; i<m_scheduledMeetingNum; i++)
		{
			energy_consumption += calculateMeetingMaxEnergy(last_meeting, m_meetingSchedule[i]);
			last_meeting = m_meetingSchedule[i];
		}
		return energy_consumption;		
	}
	
	float calculateMeetingEnergy(MeetingInfo_ptr lastMeeting, MeetingInfo_ptr currentMeeting)
	{
		int last_meeting_end_time = 0;
		
		if(lastMeeting != NULL)
		{
			last_meeting_end_time = lastMeeting->endTime();
		}
		return calculateMeetingEnergy(last_meeting_end_time, currentMeeting);
	}

	int compressSchedule()
	{
		MeetingInfo_ptr current_meeting;
		int last_end_time = MEETING_START_TIME;
		int cur_start_time, cur_end_time;
		int meeting_length;
		for(int i = 0; i< m_scheduledMeetingNum; i++)
		{
			current_meeting = m_meetingSchedule[i];
			cur_start_time = current_meeting->startTime();
			cur_end_time = current_meeting->endTime();
			meeting_length = cur_end_time - cur_start_time;

			//If have gap, move forward
			if(cur_start_time>last_end_time)
			{
				current_meeting->startTime(last_end_time);
				current_meeting->endTime(last_end_time+meeting_length);				
			}
			last_end_time = current_meeting->endTime();
		}
		return 0;
			
	}
	float calculateMeetingEnergy(int lastMeetingEndTime, MeetingInfo_ptr currentMeeting)
	{
		int current_meeting_time = (currentMeeting->endTime()- currentMeeting->startTime())*60;
		int meeting_interval = 1000;
		float recooling_time = 0;
		
		int C0 = TARGET_TEMP-OUTSIDE_TEMP;
		float E = m_effectivePower/m_lambdaA;
		float lambda = m_lambdaA/m_mC;
		
		meeting_interval = (currentMeeting->startTime()-lastMeetingEndTime)*60;
		if(meeting_interval <0)
		{
			meeting_interval = 60*60;	//Set a default value;
		}
		recooling_time = -(1/lambda)*log((C0-E)/(C0*exp(-lambda*meeting_interval) - E));
		if((recooling_time > current_meeting_time)||(recooling_time<0))
		{
			recooling_time = current_meeting_time;
		}
		
		return (recooling_time*m_ACPower + (current_meeting_time - recooling_time)*m_keepPower)/(3600*1000); //KWh 3600*1000
		
	}

#if 0
	//Calculate with outdoor temperature
	float calculateMeetingEnergyWithOT(int lastMeetingEndTime, MeetingInfo_ptr currentMeeting)
	{
		int current_meeting_time = (currentMeeting->endTime()- currentMeeting->startTime())*60;
		int meeting_interval = 1000;
		float recooling_time = 0;

		meeting_interval = (currentMeeting->startTime()-lastMeetingEndTime)*60;
		if(meeting_interval <0)
		{
			meeting_interval = 60*60;	//Set a default value;
		}

		//Calculate temerature change
		float E = exp(-lambda*meeting_interval)
		
		return (recooling_time*m_ACPower + (current_meeting_time - recooling_time)*m_keepPower)/(3600*1000); //KWh 3600*1000
		
	}
#endif

	float calculateMeetingMinEnergy(MeetingInfo_ptr lastMeeting, MeetingInfo_ptr currentMeeting)
	{
		int current_meeting_time = currentMeeting->endTime()- currentMeeting->startTime();
		//Return the minimum cost of a meeting
		return (current_meeting_time*m_keepPower*60)/(3600*1000); //KWh 3600*1000
	}

	float calculateMeetingMaxEnergy(MeetingInfo_ptr lastMeeting, MeetingInfo_ptr currentMeeting)
	{
		//Return the maximum cost of a meeting
		return calculateMeetingEnergy(0, currentMeeting);; //KWh 3600*1000
	}



	int setZoneOriginal(float x, float y, float z){m_x = x; m_y = y; m_z = z; return 0;}
	int generateZoneData(FILE* fp);
	int generateSurfaces(FILE* fp);
	int generateHVAC(FILE* fp);
	int generateZoneHVAC(FILE* fp);
	int generateHVACSchedule(FILE* fp, int flagSchedule);
	
};


#endif
